;; Generated package description from ada-mode.el
(define-package "ada-mode" "5.0.1" "major-mode for editing Ada sources" '((wisi "1.0")) :url "http://stephe-leake.org/emacs/ada-mode/emacs-ada-mode.html" :keywords 'nil)
